#define SERVO_PIN     PIN_E3

#define TIMER_1_DIV     2

//
// You will likely need to calibrate these values to different values depending
// on the servo you are using.
//
#define SHORT_TIME     0.00063  // Shortest pulse width high time
#define CENTER_TIME    0.0015   // The high time for center
#define LONG_TIME      0.00242  // Longest pulse width high time
#define PULSE_TIME     0.0200   // The total time for the pulse

#define TIMER_RATE      getenv("CLOCK") / 4 / TIMER_1_DIV

#define SHORT_TICKS     (int16)((float)TIMER_RATE * SHORT_TIME)
#define CENTER_TICKS    (int16)((float)TIMER_RATE * CENTER_TIME)
#define LONG_TICKS      (int16)((float)TIMER_RATE * LONG_TIME)
#define LOW_TICKS       (int16)((float)TIMER_RATE * PULSE_TIME)
#define DEGREE_TICKS    (int16)((float)TIMER_RATE * (LONG_TIME - SHORT_TIME) / 180.f)

//
// Globals Varibles
//
int16 gServoTicks = 0;
int1  gStopServo = TRUE;

///////////////////////////////////////////////////////////////////////////

void initServo()
{
   #if TIMER_1_DIV == 1
      setup_timer_1(T1_DIV_BY_1 | T1_INTERNAL);
   #elif TIMER_1_DIV == 2
      setup_timer_1(T1_DIV_BY_2 | T1_INTERNAL);
   #elif TIMER_1_DIV == 4
      setup_timer_1(T1_DIV_BY_4 | T1_INTERNAL);
   #elif TIMER_1_DIV == 8
      setup_timer_1(T1_DIV_BY_8 | T1_INTERNAL);
   #endif

   setup_ccp1(CCP_COMPARE_INT);

   gServoTicks = 0;
   gStopServo = FALSE;

   enable_interrupts(INT_CCP1);
   enable_interrupts(GLOBAL);
}


//
// position: -90 to 90 degrees
//
void setServo(signed int8 position)
{
    gServoTicks = (int16)((signed int16)position + 90) * DEGREE_TICKS + SHORT_TICKS;
}


// Purpose:    Stop the servos from moving,
//             but remember their speed settings
void pauseServos()
{
   gStopServo = TRUE;
}


// Purpose:    Resume the servos after a pause
void resumeServos()
{
   gStopServo = FALSE;
}

#int_ccp1
void isr_ccp1()
{
   static int1 low = TRUE;

   if(gStopServo == FALSE)
   {
      if(low)
      {
         output_low(SERVO_PIN);             // Set the servo control pin to low
         CCP_1 += LOW_TICKS - gServoTicks;  // Set CCP1 to interrupt for next high pulse
         low = FALSE;
      }
      else
      {
         output_high(SERVO_PIN);            // Set the servo control pin to high
         CCP_1 += gServoTicks;              // Set CCP1 to interrupt for next low pulse
         low = TRUE;
      }
   }
}


